@@ -1,7 +1,7 @@
 /*******************************************************************************
    This file is part of SecureSkat.
 
- Copyright (C) 2007, 2017  Heiko Stamer <HeikoStamer@gmx.net>
+ Copyright (C) 2007, 2017, 2018  Heiko Stamer <HeikoStamer@gmx.net>
 
    SecureSkat is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -68,7 +68,7 @@ int skat_connect
 		// send the signature
 		*neighbor << sec.sign(ost.str()) << std::endl << std::flush;
 		// create a fresh nonce
-		nonce_B = mpz_srandom_ui();
+		nonce_B = tmcg_mpz_srandom_ui();
 		// send challenge
 		*neighbor << nonce_B << std::endl << std::flush;
 		// receive the signature
@@ -154,7 +154,7 @@ int skat_accept
 			{
 				iosocketstream *neighbor = new iosocketstream(handle);
 				// create a fresh nonce
-				unsigned long int nonce_A = mpz_srandom_ui(), nonce_B = 0;
+				unsigned long int nonce_A = tmcg_mpz_srandom_ui(), nonce_B = 0;
 				std::ostringstream ost, ost2;
 				std::string tmp;
 				*neighbor << nonce_A << std::endl << std::flush;
@@ -282,6 +282,8 @@ int skat_accept
 int skat_alive
 	(iosecuresocketstream *r, iosecuresocketstream *l)
 {
+	if ((r == NULL) || (l == NULL))
+		return -55;
 	if (!r->good() || !l->good())
 		return -5;
 	return 0;
@@ -483,6 +485,7 @@ int skat_child
 		
 		if (cmd.find("!KICK", 0) == 0)
 		{
+			CloseHandle(gp_handle);
 			delete gp_tmcg;
 			delete [] ipipe_readbuf;
 			delete in_pipe, delete out_pipe;
@@ -508,6 +511,7 @@ int skat_child
 			if (std::find(vnicks.begin(), vnicks.end(), nick) != vnicks.end())
 			{
 				*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
+				CloseHandle(gp_handle);
 				delete gp_tmcg;
 				delete [] ipipe_readbuf;
 				delete in_pipe, delete out_pipe;
@@ -530,8 +534,8 @@ int skat_child
 		}
 	}
 	std::cout << X << _("Table") << " " << nr << " " << _("establishing secure channels") << " ..." << std::endl;
-	int connect_handle, accept_handle, error = 0;
-	iosecuresocketstream *left_neighbor, *right_neighbor;
+	int connect_handle = -1, accept_handle = -1, error = 0;
+	iosecuresocketstream *left_neighbor = NULL, *right_neighbor = NULL;
 	switch (pkr_self)
 	{
 		case 0:
@@ -630,8 +634,10 @@ int skat_child
 	
 	// release the game
 	delete left_neighbor, delete right_neighbor;
-	CloseHandle(connect_handle);
-	CloseHandle(accept_handle);
+	if (connect_handle > 0)
+		CloseHandle(connect_handle);
+	if (accept_handle > 0)
+		CloseHandle(accept_handle);
 	if (exit_code != 6)
 		*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
 	CloseHandle(gp_handle);
