@@ -343,34 +343,6 @@ int ballot_child
 			}
 			else
 			{
-				// check host address
-				struct sockaddr_in sin;
-				struct hostent *hostinf = gethostbyname(nick_players[vnicks[pkr_idx]].c_str());
-				if (hostinf != NULL)
-				{
-					memcpy((char*)&sin.sin_addr, hostinf->h_addr, hostinf->h_length);
-				}
-				else
-				{
-					perror("ballot_child (gethostbyname)");
-					*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
-					delete in_pipe, delete out_pipe;
-					delete ballot_tmcg;
-					CloseHandle(gp_handle);
-					delete ireadbuf;
-					CloseHandle(handle);
-					return -70;
-				}
-				if (client_in.sin_addr.s_addr != sin.sin_addr.s_addr)
-				{
-					*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
-					delete in_pipe, delete out_pipe;
-					delete ballot_tmcg;
-					CloseHandle(gp_handle);
-					delete ireadbuf;
-					CloseHandle(handle);
-					return -71;
-				}
 				// establish and authenticate the connection
 				iosocketstream *neighbor = new iosocketstream(handle);
 				TMCG_CardSecret cs(gp_nick.size(), b);
@@ -386,7 +358,7 @@ int ballot_child
 					delete in_pipe, delete out_pipe;
 					delete ballot_tmcg;
 					CloseHandle(gp_handle);
-					delete ireadbuf;
+					delete [] ireadbuf;
 					CloseHandle(handle);
 					delete neighbor;
 					return -72;
@@ -397,7 +369,7 @@ int ballot_child
 					delete in_pipe, delete out_pipe;
 					delete ballot_tmcg;
 					CloseHandle(gp_handle);
-					delete ireadbuf;
+					delete [] ireadbuf;
 					CloseHandle(handle);
 					delete neighbor;
 					return -73;
@@ -419,12 +391,14 @@ int ballot_child
 						if (!sec.decrypt(dv, challenge_sig))
 						{
 							std::cerr << _("TMCG: decrypt() failed") << std::endl;
-							delete [] key1, delete [] key2, delete [] dv;
+							delete [] key1;
+							delete [] key2;
+							delete [] dv;
 							*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
 							delete in_pipe, delete out_pipe;
 							delete ballot_tmcg;
 							CloseHandle(gp_handle);
-							delete ireadbuf;
+							delete [] ireadbuf;
 							CloseHandle(handle);
 							delete neighbor;
 							return -74;
@@ -432,11 +406,15 @@ int ballot_child
 						memcpy(key2, dv, TMCG_SAEP_S0);
 						gcry_randomize(key1, TMCG_SAEP_S0, GCRY_STRONG_RANDOM);
 						*neighbor << pkr.keys[pkr_idx].encrypt(key1) << std::endl << std::flush;
+						delete neighbor;
+						// create encrypted stream and release keys
 						ios_in[vnicks[pkr_idx]] = new iosecuresocketstream(handle, key1, 16, key2, 16);
 #ifndef NDEBUG
 						std::cerr << "ios_in[" << vnicks[pkr_idx] << "]" << std::endl;
 #endif
-						delete neighbor, delete [] key1, delete [] key2, delete [] dv;						
+						delete [] key1;
+						delete [] key2;
+						delete [] dv;						
 						pkr_idx++;
 					}
 					else
@@ -445,14 +423,13 @@ int ballot_child
 						delete in_pipe, delete out_pipe;
 						delete ballot_tmcg;
 						CloseHandle(gp_handle);
-						delete ireadbuf;
+						delete [] ireadbuf;
 						CloseHandle(handle);
 						delete neighbor;
 						return -76;
 					}
 				}
 			}
-			CloseHandle(handle);
 		}
 		else if ((ret > 0) && FD_ISSET(ipipe, &rfds))
 		{
@@ -479,7 +456,7 @@ int ballot_child
 						delete in_pipe, delete out_pipe;
 						delete ballot_tmcg;
 						CloseHandle(gp_handle);
-						delete ireadbuf;
+						delete [] ireadbuf;
 						return -1;
 					}
 					if (neu && (cmd.find("!ANNOUNCE", 0) == 0))
@@ -501,7 +478,7 @@ int ballot_child
 							delete in_pipe, delete out_pipe;
 							delete ballot_tmcg;
 							CloseHandle(gp_handle);
-							delete ireadbuf;
+							delete [] ireadbuf;
 							return -77;
 						}
 					}
@@ -523,7 +500,7 @@ int ballot_child
 				delete in_pipe, delete out_pipe;
 				delete ballot_tmcg;
 				CloseHandle(gp_handle);
-				delete ireadbuf;
+				delete [] ireadbuf;
 				return -78;
 			}
 			else if (num < 0)
@@ -548,7 +525,7 @@ int ballot_child
 							delete in_pipe, delete out_pipe;
 							delete ballot_tmcg;
 							CloseHandle(gp_handle);
-							delete ireadbuf;
+							delete [] ireadbuf;
 							return -79;
 						}
 						iosocketstream *neighbor = new iosocketstream(handle);
@@ -578,7 +555,7 @@ int ballot_child
 								delete in_pipe, delete out_pipe;
 								delete ballot_tmcg;
 								CloseHandle(gp_handle);
-								delete ireadbuf;
+								delete [] ireadbuf;
 								CloseHandle(handle);
 								delete neighbor;
 								return -80;
@@ -589,7 +566,7 @@ int ballot_child
 								delete in_pipe, delete out_pipe;
 								delete ballot_tmcg;
 								CloseHandle(gp_handle);
-								delete ireadbuf;
+								delete [] ireadbuf;
 								CloseHandle(handle);
 								delete neighbor;
 								return -81;
@@ -601,7 +578,7 @@ int ballot_child
 							delete in_pipe, delete out_pipe;
 							delete ballot_tmcg;
 							CloseHandle(gp_handle);
-							delete ireadbuf;
+							delete [] ireadbuf;
 							CloseHandle(handle);
 							delete neighbor;
 							return -82;
@@ -618,131 +595,95 @@ int ballot_child
 						if (!sec.decrypt(dv, tmp))
 						{
 							std::cerr << _("TMCG: decrypt() failed") << std::endl;
-							delete [] key1, delete [] key2, delete [] dv;
+							delete [] key1;
+							delete [] key2;
+							delete [] dv;
 							*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
 							delete in_pipe, delete out_pipe;
 							delete ballot_tmcg;
 							CloseHandle(gp_handle);
-							delete ireadbuf;
+							delete [] ireadbuf;
 							CloseHandle(handle);
 							delete neighbor;
 							return -84;
 						}
 						memcpy(key2, dv, TMCG_SAEP_S0);
-						ios_out[vnicks[i]] = 
-							new iosecuresocketstream(handle, key1, 16, key2, 16);
+						delete neighbor;
+						// create encrypted stream and release keys
+						ios_out[vnicks[i]] = new iosecuresocketstream(handle, key1, 16, key2, 16);
 #ifndef NDEBUG
 						std::cerr << "ios_out[" << vnicks[i] << "]" << std::endl;
 #endif
-						delete neighbor, delete [] key1, delete [] key2, delete [] dv;
-						CloseHandle(handle);
+						delete [] key1;
+						delete [] key2;
+						delete [] dv;
 					}
 				}
 				pkr_idx++;
 			}
 		}
 	} // while
-	delete ireadbuf;
-	
+	delete [] ireadbuf;
+
 	// VTMF initialization
 	BarnettSmartVTMF_dlog *vtmf;
 	if (pkr_self == 0)
 	{
-		vtmf = new BarnettSmartVTMF_dlog;
-		for (size_t i = 0; i < vnicks.size(); i++)
-		{
-			if (i != pkr_self)
-				vtmf->PublishGroup(*ios_out[vnicks[i]]);
-		}
-		if (!vtmf->CheckGroup())
-		{
-			std::cout << ">< " << _("VTMF ERROR") << ": " << _("function CheckGroup() failed") << std::endl;
-			*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
-			delete vtmf;
-			for (size_t ii = 0; ii < vnicks.size(); ii++)
-				delete ios_out[vnicks[ii]];
-			delete in_pipe, delete out_pipe;
-			delete ballot_tmcg;
-			CloseHandle(gp_handle);
-			return -90;
-		}
-		vtmf->KeyGenerationProtocol_GenerateKey();
-		for (size_t i = 0; i < vnicks.size(); i++)
-		{
-			if (i != pkr_self)
-			{
-				if (!vtmf->KeyGenerationProtocol_UpdateKey(*ios_in[vnicks[i]]))
-				{
-					std::cout << ">< " << _("VTMF ERROR") << ": " << _("function KeyGenerationProtocol_UpdateKey() failed") << 
-						" " << _("for") << " " << vnicks[i]<< std::endl;
-					*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
-					delete vtmf;
-					for (size_t ii = 0; ii < vnicks.size(); ii++)
-						delete ios_out[vnicks[ii]];
-					delete in_pipe, delete out_pipe;
-					delete ballot_tmcg;
-					CloseHandle(gp_handle);
-					return -90;
-				}
-			}
-			else
-			{
-				for (size_t j = 0; j < vnicks.size(); j++)
-				{
-					if (j != pkr_self)
-						vtmf->KeyGenerationProtocol_PublishKey(*ios_out[vnicks[j]]);
-				}
-			}
-		}
-		vtmf->KeyGenerationProtocol_Finalize();
+		vtmf = new BarnettSmartVTMF_dlog();
+		for (size_t i = 1; i < vnicks.size(); i++)
+			vtmf->PublishGroup(*ios_out[vnicks[i]]);
 	}
 	else
 	{
 		vtmf = new BarnettSmartVTMF_dlog(*ios_in[vnicks[0]]);
-		
-		if (!vtmf->CheckGroup())
-		{
-			std::cout << ">< " << _("VTMF ERROR") << ": " << _("function CheckGroup() failed") << std::endl;
-			*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
-			delete vtmf;
-			for (size_t ii = 0; ii < vnicks.size(); ii++)
-				delete ios_out[vnicks[ii]];
-			delete in_pipe, delete out_pipe;
-			delete ballot_tmcg;
-			CloseHandle(gp_handle);
-			return -90;
-		}
-		
-		vtmf->KeyGenerationProtocol_GenerateKey();
-		for (size_t i = 0; i < vnicks.size(); i++)
-		{
-			if (i != pkr_self)
-			{
-				if (!vtmf->KeyGenerationProtocol_UpdateKey(*ios_in[vnicks[i]]))
-				{
-					std::cout << ">< " << _("VTMF ERROR") << ": " << _("function KeyGenerationProtocol_UpdateKey() failed") << 
-						" " << _("for") << " " << vnicks[i]<< std::endl;
-					*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
-					delete vtmf;
-					for (size_t ii = 0; ii < vnicks.size(); ii++)
-						delete ios_out[vnicks[ii]];
-					delete in_pipe, delete out_pipe;
-					delete ballot_tmcg;
-					CloseHandle(gp_handle);
-					return -90;
-				}
-			}
-			else
-			{
-				for (size_t j = 0; j < vnicks.size(); j++)
-				{
-					if (j != pkr_self)
-						vtmf->KeyGenerationProtocol_PublishKey(*ios_out[vnicks[j]]);
-				}
-			}
-		}
-		vtmf->KeyGenerationProtocol_Finalize();
 	}
+	if (!vtmf->CheckGroup())
+	{
+		std::cerr << ">< " << _("VTMF ERROR") << ": " << _("function CheckGroup() failed") << std::endl;
+		*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
+		delete vtmf;
+		for (size_t ii = 0; ii < vnicks.size(); ii++)
+			delete ios_out[vnicks[ii]];
+		delete in_pipe, delete out_pipe;
+		delete ballot_tmcg;
+		CloseHandle(gp_handle);
+		return -90;
+	}
+	vtmf->KeyGenerationProtocol_GenerateKey();
+	for (size_t i = 0; i < vnicks.size(); i++)
+	{
+		if (i != pkr_self)
+		{
+			if (!vtmf->KeyGenerationProtocol_UpdateKey(*ios_in[vnicks[i]]))
+			{
+				std::cerr << ">< " << _("VTMF ERROR") << ": " << _("function KeyGenerationProtocol_UpdateKey() failed") << 
+					" " << _("for") << " " << vnicks[i] << std::endl;
+				*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
+				delete vtmf;
+				for (size_t ii = 0; ii < vnicks.size(); ii++)
+				{
+					if (ii != pkr_self)
+					{
+						delete ios_in[vnicks[ii]];
+						delete ios_out[vnicks[ii]];
+					}
+				}
+				delete in_pipe, delete out_pipe;
+				delete ballot_tmcg;
+				CloseHandle(gp_handle);
+				return -90;
+			}
+		}
+		else
+		{
+			for (size_t j = 0; j < vnicks.size(); j++)
+			{
+				if (j != pkr_self)
+					vtmf->KeyGenerationProtocol_PublishKey(*ios_out[vnicks[j]]);
+			}
+		}
+	}
+	vtmf->KeyGenerationProtocol_Finalize();
 	
 	// create private card for ballot
 	VTMF_Card vote_c;
@@ -768,7 +709,13 @@ int ballot_child
 				*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
 				delete vtmf;
 				for (size_t ii = 0; ii < vnicks.size(); ii++)
-					delete ios_out[vnicks[ii]];
+				{
+					if (ii != pkr_self)
+					{
+						delete ios_in[vnicks[ii]];
+						delete ios_out[vnicks[ii]];
+					}
+				}
 				delete in_pipe, delete out_pipe;
 				delete ballot_tmcg;
 				CloseHandle(gp_handle);
@@ -811,7 +758,13 @@ int ballot_child
 					*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
 					delete vtmf;
 					for (size_t ii = 0; ii < vnicks.size(); ii++)
-						delete ios_out[vnicks[ii]];
+					{
+						if (ii != pkr_self)
+						{
+							delete ios_in[vnicks[ii]];
+							delete ios_out[vnicks[ii]];
+						}
+					}
 					delete in_pipe, delete out_pipe;
 					delete ballot_tmcg;
 					CloseHandle(gp_handle);
@@ -824,7 +777,13 @@ int ballot_child
 				*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
 				delete vtmf;
 				for (size_t ii = 0; ii < vnicks.size(); ii++)
-					delete ios_out[vnicks[ii]];
+				{
+					if (ii != pkr_self)
+					{
+						delete ios_in[vnicks[ii]];
+						delete ios_out[vnicks[ii]];
+					}
+				}
 				delete in_pipe, delete out_pipe;
 				delete ballot_tmcg;
 				CloseHandle(gp_handle);
@@ -864,7 +823,13 @@ int ballot_child
 					*out_pipe << "PART " << MAIN_CHANNEL_UNDERSCORE << nr << std::endl << std::flush;
 					delete vtmf;
 					for (size_t ii = 0; ii < vnicks.size(); ii++)
-						delete ios_out[vnicks[ii]];
+					{
+						if (ii != pkr_self)
+						{
+							delete ios_in[vnicks[ii]];
+							delete ios_out[vnicks[ii]];
+						}
+					}
 					delete in_pipe, delete out_pipe;
 					delete ballot_tmcg;
 					CloseHandle(gp_handle);
@@ -901,7 +866,13 @@ int ballot_child
 	
 	// exit from room
 	for (size_t ii = 0; ii < vnicks.size(); ii++)
-		delete ios_out[vnicks[ii]];
+	{
+		if (ii != pkr_self)
+		{
+			delete ios_in[vnicks[ii]];
+			delete ios_out[vnicks[ii]];
+		}
+	}
 	CloseHandle(gp_handle);
 	delete vtmf;
 	delete ballot_tmcg;
